package main

import (
	"encoding/json"
	"fmt"
	"slices"
	"strings"

	"src.opensuse.org/autogits/common"
)

type OrgLinks struct {
	Source string // org/pkg#branch
	Target string // pkg#branch

	SOid, TOid string // source and target oids
}

func FetchProjectLinksFile(org, prj string) ([]byte, error) {
	return nil, nil
}

func ParseProjectLinks(data []byte) ([]*OrgLinks, error) {
	values := make([]*OrgLinks, 0, 100)

	if len(data) == 0 {
		return values, nil
	}

	if err := json.Unmarshal(data, &values); err != nil {
		return nil, err
	}

	slices.SortFunc(values, func(a, b *OrgLinks) int {
		return strings.Compare(a.Target, b.Target)
	})

	for _, link := range values {
		if len(link.Source) < 5 {
			return nil, fmt.Errorf("Invalid Source: %s", link.Source)
		}
		source_link := strings.Split(link.Source, "/")
		if len(source_link) != 2 || len(source_link[0]) < 1 || len(source_link[1]) < 3 {
			return nil, fmt.Errorf("Invalid Source: %s", link.Source)
		}
		branch := strings.Split(source_link[1], "#")
		if len(branch) != 2 || len(branch[0]) < 1 || len(branch[1]) < 1 {
			return nil, fmt.Errorf("Invalid Source: %s", link.Source)
		}
		target_link := strings.Split(link.Target, "#")
		if len(target_link) != 2 || len(target_link[0]) < 1 || len(target_link[1]) < 1 {
			return nil, fmt.Errorf("Missing Target for Source link: %s", link.Source)
		}
	}

	return values, nil
}

func (link *OrgLinks) StateUpdater(git common.Git, gitea common.Gitea) {
}

